@@ -53,6 +53,7 @@
 #include "gpm-dpms.h"
 #include "gpm-idle.h"
 #include "gpm-power.h"
+#include "gpm-brightness.h"
 #include "gpm-tray-icon.h"
 #include "gpm-manager.h"
 
@@ -72,8 +73,15 @@ struct GpmManagerPrivate
 	GpmDpms		*dpms;
 	GpmIdle		*idle;
 	GpmPower	*power;
+	GpmBrightness   *brightness;
 
 	GpmTrayIcon	*tray_icon;
+
+	gboolean	 done_warning_critical;
+	gboolean	 done_warning_very_low;
+	gboolean	 done_warning_low;
+
+	gboolean	 use_time_to_notify;
 };
 
 enum {
@@ -89,60 +97,96 @@ enum {
 static GObjectClass *parent_class = NULL;
 static guint	     signals [LAST_SIGNAL] = { 0, };
 
+static GConfEnumStringPair icon_policy_enum_map [] = {
+       { GPM_ICON_POLICY_ALWAYS,       "always"   },
+       { GPM_ICON_POLICY_CHARGE,       "charge"   },
+       { GPM_ICON_POLICY_CRITICAL,     "critical" },
+       { GPM_ICON_POLICY_NEVER,        "never"    },
+       { 0, NULL }
+};
+
 G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
 
+/* FIXME */
+#define		BATTERY_LOW_PERCENTAGE			(10)	  /* 10 percent */
+#define		BATTERY_VERY_LOW_PERCENTAGE		(5)	  /* 5 percent  */
+#define		BATTERY_CRITICAL_PERCENTAGE		(2)	  /* 2 percent  */
+#define		BATTERY_LOW_REMAINING_TIME		(20 * 60) /* 20 minutes */
+#define		BATTERY_VERY_LOW_REMAINING_TIME		(10 * 60) /* 10 minutes */
+#define		BATTERY_CRITICAL_REMAINING_TIME		(5 * 60)  /* 5 minutes  */
+
 #undef DISABLE_ACTIONS_FOR_TESTING
 /*#define DISABLE_ACTIONS_FOR_TESTING 1*/
 
-static gboolean
-_gpm_manager_can_suspend (GpmManager *manager)
+gboolean
+gpm_manager_can_suspend (GpmManager *manager,
+			 gboolean   *can,
+			 GError    **error)
 {
-	gboolean can;
+	gboolean gconf_policy;
+	g_return_val_if_fail (can, FALSE);
+
+	*can = FALSE;
 
 #ifdef DISABLE_ACTIONS_FOR_TESTING
 	g_debug ("Suspend disabled for testing");
-	return FALSE;
+	return TRUE;
 #endif
 
-	/* FIXME: check other stuff */
+	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+					      GPM_PREF_CAN_SUSPEND, NULL);
+	if ( gconf_policy && gpm_hal_can_suspend () ) {
+		*can = TRUE;
+	}
 
-	can = gpm_hal_can_suspend ();
-
-	return can;
+	return TRUE;
 }
 
-static gboolean
-_gpm_manager_can_hibernate (GpmManager *manager)
+gboolean
+gpm_manager_can_hibernate (GpmManager *manager,
+			   gboolean   *can,
+			   GError    **error)
 {
-	gboolean can;
+	gboolean gconf_policy;
+	g_return_val_if_fail (can, FALSE);
+
+	*can = FALSE;
 
 #ifdef DISABLE_ACTIONS_FOR_TESTING
 	g_debug ("Hibernate disabled for testing");
-	return FALSE;
+	return TRUE;
 #endif
 
-	/* FIXME: check other stuff */
+	gconf_policy = gconf_client_get_bool (manager->priv->gconf_client,
+					      GPM_PREF_CAN_HIBERNATE, NULL);
+	if ( gconf_policy && gpm_hal_can_hibernate () ) {
+		*can = TRUE;
+	}
 
-	can = gpm_hal_can_hibernate ();
-
-	return can;
+	return TRUE;
 }
 
-static gboolean
-_gpm_manager_can_shutdown (GpmManager *manager)
+gboolean
+gpm_manager_can_shutdown (GpmManager *manager,
+			  gboolean   *can,
+			  GError    **error)
 {
-	gboolean can;
+	if (can) {
+		*can = FALSE;
+	}
 
 #ifdef DISABLE_ACTIONS_FOR_TESTING
 	g_debug ("Shutdown disabled for testing");
-	return FALSE;
+	return TRUE;
 #endif
 
 	/* FIXME: check other stuff */
 
-	can = TRUE;
+	if (can) {
+		*can = TRUE;
+	}
 
-	return can;
+	return TRUE;
 }
 
 
@@ -172,85 +216,88 @@ get_icon_index_from_percent (gint percent)
  */
 static char *
 get_stock_id (GpmManager *manager,
-	      char	 *icon_policy)
+	      int         icon_policy)
 {
 	gboolean res;
 	gboolean has_primary;
 	int	 index;
-	int	 low_threshold;
 	int      primary_percentage;
 	int      percentage;
 	gboolean primary_charging;
 	gboolean primary_discharging;
 	gboolean on_ac;
+	char    *stock_id;
 
-	g_return_val_if_fail (icon_policy != NULL, NULL);
+	stock_id = NULL;
 
 	g_debug ("Getting stock icon for tray");
 
-	if (strcmp (icon_policy, ICON_POLICY_NEVER) == 0) {
+	if (icon_policy == GPM_ICON_POLICY_NEVER) {
 		g_debug ("The key " GPM_PREF_ICON_POLICY
 			 " is set to never, so no icon will be displayed.\n"
 			 "You can change this using gnome-power-preferences");
-		return NULL;
+		goto done;
 	}
 
-	/* find out when the user considers the power "low" */
-	low_threshold = gconf_client_get_int (manager->priv->gconf_client, GPM_PREF_THRESHOLD_LOW, NULL);
-
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
 	has_primary = gpm_power_get_battery_percentage (manager->priv->power,
 							"primary",
 							&primary_percentage,
 							NULL);
-	if (has_primary && primary_percentage < low_threshold) {
+	if (has_primary && primary_percentage < BATTERY_LOW_PERCENTAGE) {
 		index = get_icon_index_from_percent (primary_percentage);
 
 		if (on_ac) {
-			return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
+			stock_id = g_strdup_printf ("gnome-power-ac-%d-of-8", index);
 		} else {
-			return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
+			stock_id = g_strdup_printf ("gnome-power-bat-%d-of-8", index);
 		}
+
+		goto done;
 	}
 
 	res = gpm_power_get_battery_percentage (manager->priv->power,
 						"ups",
 						&percentage,
 						NULL);
-	if (res && percentage < low_threshold) {
+	if (res && percentage < BATTERY_LOW_PERCENTAGE) {
 		index = get_icon_index_from_percent (percentage);
 
-		return g_strdup_printf ("gnome-power-ups-%d-of-8", index);
+		stock_id = g_strdup_printf ("gnome-power-ups-%d-of-8", index);
+		goto done;
 	}
 
 	res = gpm_power_get_battery_percentage (manager->priv->power,
 						"mouse",
 						&percentage,
 						NULL);
-	if (res && percentage < low_threshold) {
-		return g_strdup_printf ("gnome-power-mouse");
+	if (res && percentage < BATTERY_LOW_PERCENTAGE) {
+		stock_id = g_strdup_printf ("gnome-power-mouse");
+		goto done;
 	}
 
 	res = gpm_power_get_battery_percentage (manager->priv->power,
 						"keyboard",
 						&percentage,
 						NULL);
-	if (res && percentage < low_threshold) {
-		return g_strdup_printf ("gnome-power-keyboard");
+	if (res && percentage < BATTERY_LOW_PERCENTAGE) {
+		stock_id = g_strdup_printf ("gnome-power-keyboard");
+		goto done;
 	}
 
 	/*
 	 * Check if we should just show the charging / discharging icon 
 	 * even when not low or critical.
 	 */
-	if ((strcmp (icon_policy, ICON_POLICY_CRITICAL) == 0)) {
+	if (icon_policy == GPM_ICON_POLICY_CRITICAL) {
 		g_debug ("get_stock_id: no devices critical, so "
 			 "no icon will be displayed.");
-		return NULL;
+		stock_id = NULL;
+		goto done;
 	}
 
-	/* Only display if charging or disharging */
+	/* Only display if charging or discharging */
 	primary_charging = FALSE;
 	primary_discharging = FALSE;
 	if (has_primary) {
@@ -259,21 +306,24 @@ get_stock_id (GpmManager *manager,
 						      &primary_charging,
 						      &primary_discharging,
 						      NULL);
+
 		if (primary_charging || primary_discharging) {
 			index = get_icon_index_from_percent (primary_percentage);
 			if (on_ac) {
-				return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
+				stock_id = g_strdup_printf ("gnome-power-ac-%d-of-8", index);
 			} else {
-				return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
+				stock_id = g_strdup_printf ("gnome-power-bat-%d-of-8", index);
 			}
+			goto done;
 		}
 	}
 
 	/* Check if we should just show the icon all the time */
-	if (strcmp (icon_policy, ICON_POLICY_CHARGE) == 0) {
+	if (icon_policy == GPM_ICON_POLICY_CHARGE) {
 		g_debug ("get_stock_id: no devices (dis)charging, so "
 			 "no icon will be displayed.");
-		return NULL;
+		stock_id = NULL;
+		goto done;
 	}
 
 	/* Do the rest of the battery icon states */
@@ -282,43 +332,48 @@ get_stock_id (GpmManager *manager,
 
 		if (on_ac) {
 			if (!primary_charging && !primary_discharging) {
-				return g_strdup ("gnome-power-ac-charged");
+				stock_id = g_strdup ("gnome-power-ac-charged");
 			} else {
-				return g_strdup_printf ("gnome-power-ac-%d-of-8", index);
+				stock_id = g_strdup_printf ("gnome-power-ac-%d-of-8", index);
 			}
+
+			goto done;
 		} else {
-			return g_strdup_printf ("gnome-power-bat-%d-of-8", index);
+			stock_id = g_strdup_printf ("gnome-power-bat-%d-of-8", index);
+			goto done;
 		}
 	}
 
 	/* We fallback to the ac_adapter icon */
-	return g_strdup_printf ("gnome-dev-acadapter");
+	stock_id = g_strdup_printf ("gnome-dev-acadapter");
+
+ done:
+	g_debug ("Going to use stock id: %s", stock_id);
+
+	return stock_id;
 }
 
 static void
 tray_icon_update (GpmManager *manager)
 {
 	char *stock_id = NULL;
-	char *icon_policy;
+	char *icon_policy_str;
+	int   icon_policy;
 
 	/* do we want to display the icon */
-	icon_policy = gconf_client_get_string (manager->priv->gconf_client, GPM_PREF_ICON_POLICY, NULL);
-
-	if (! icon_policy) {
-		g_warning ("You have not set an icon policy! "
-			   "(Please run gnome-power-preferences) -- "
-			   "I'll assume you want an icon all the time...");
-		icon_policy = g_strdup (ICON_POLICY_ALWAYS);
-	}
+	icon_policy_str = gconf_client_get_string (manager->priv->gconf_client, GPM_PREF_ICON_POLICY, NULL);
+	icon_policy = GPM_ICON_POLICY_ALWAYS;
+	gconf_string_to_enum (icon_policy_enum_map, icon_policy_str, &icon_policy);
+	g_free (icon_policy_str);
 
 	/* try to get stock image */
 	stock_id = get_stock_id (manager, icon_policy);
-	g_free (icon_policy);
 
 	/* only create if we have a valid filename */
 	if (stock_id) {
 		char *tooltip = NULL;
 
+		/* make sure that we have a valid object */
 		if (! manager->priv->tray_icon) {
 			gpm_manager_setup_tray_icon (manager, NULL);
 		}
@@ -436,7 +491,7 @@ change_power_policy (GpmManager *manager,
 		sleep_display = gconf_client_get_int (client, GPM_PREF_BATTERY_SLEEP_DISPLAY, NULL);
 	}
 
-	gpm_hal_set_brightness_dim (brightness);
+	gpm_brightness_level_dim (manager->priv->brightness, brightness);
 	gpm_hal_enable_power_save (!on_ac);
 
 	/*
@@ -456,13 +511,17 @@ maybe_notify_on_ac_changed (GpmManager *manager,
 {
 	gboolean show_notify;
 
-	if (manager->priv->tray_icon == NULL) {
-		return;
-	}
-
 	show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 					     GPM_PREF_NOTIFY_ACADAPTER, NULL);
 
+	/* update icon */
+	tray_icon_update (manager);
+
+	/* If no tray icon then don't notify */
+	if (! manager->priv->tray_icon) {
+		return;
+	}
+
 	if (! on_ac) {
 		if (show_notify) {
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
@@ -473,16 +532,12 @@ maybe_notify_on_ac_changed (GpmManager *manager,
 					       "The system is now using battery power."));
 		}
 	} else {
-
 		/*
 		 * for where we add back the ac_adapter before
 		 * the "AC Power unplugged" message times out.
 		 */
 		gpm_tray_icon_cancel_notify (GPM_TRAY_ICON (manager->priv->tray_icon));
 	}
-
-	/* update icon */
-	tray_icon_update (manager);
 }
 
 /** Do the action dictated by policy from gconf
@@ -523,7 +578,7 @@ manager_policy_do (GpmManager *manager,
 		gpm_manager_shutdown (manager);
 
 	} else {
-		g_warning ("manager_policy_do called with unknown action %s", action);
+		g_warning ("manager_policy_do: called with unknown action %s", action);
 	}
 
 	g_free (action);
@@ -534,31 +589,30 @@ static void
 maybe_notify_battery_power_changed (GpmManager         *manager,
 				    const char         *kind,
 				    int		        percentage,
-				    glong	        minutes,
+				    int	        	remaining_time,
 				    gboolean	        discharging,
 				    gboolean	        charging,
 				    gboolean	        percentagechanged)
 {
 	gboolean show_notify;
 	gboolean primary;
-	gint	 low_threshold;
-	gint	 critical_threshold;
 	gchar	*message;
 	gchar	*remaining;
+	gboolean warning_low = FALSE;
+	gboolean warning_very_low = FALSE;
+	gboolean warning_critical = FALSE;
 
 	primary = (strcmp (kind, "primary") == 0);
 
-	g_debug ("percentage = %d, minutes = %ld, discharging = %d, "
+	g_debug ("percentage = %d, remaining_time = %d, discharging = %d, "
 		 "charging = %d, primary = %d, percentagechanged=%i",
-		 percentage, minutes, discharging, charging, primary, percentagechanged);
+		 percentage, remaining_time, discharging, charging, primary, percentagechanged);
 
-	if (manager->priv->tray_icon == NULL) {
-		return;
+	/* If no tray icon then don't notify */
+	if (! manager->priv->tray_icon) {
+		goto done;
 	}
 
-	/* update icon */
-	tray_icon_update (manager);
-
 	/* give notification @100%, on percentagechanged */
 	if (percentagechanged && primary && percentage >= 100) {
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
@@ -572,40 +626,59 @@ maybe_notify_battery_power_changed (GpmManager         *manager,
 					      _("Your battery is now fully charged"));
 		}
 
-		return;
+		goto done;
 	}
 
-	if (! discharging) {
-		g_debug ("battery is not discharging!");
-		return;
+	if (! discharging || ! primary) {
+		g_debug ("maybe_notify_battery_power_changed: Primary battery is not discharging!");
+		goto done;
 	}
 
-	low_threshold = gconf_client_get_int (manager->priv->gconf_client,
-					      GPM_PREF_THRESHOLD_LOW, NULL);
-	critical_threshold = gconf_client_get_int (manager->priv->gconf_client,
-						   GPM_PREF_THRESHOLD_CRITICAL, NULL);
+	g_debug ("percentage = %d, remaining_time = %i", percentage, remaining_time);
 
-	g_debug ("percentage = %d, low_threshold = %i, critical_threshold = %i",
-		 percentage, low_threshold, critical_threshold);
+	/* this is for compatability. If the time stuff doesn't work out then
+	 * we can easily add back support like it was */
+	if (! manager->priv->use_time_to_notify) {
+		if (percentage <= BATTERY_CRITICAL_PERCENTAGE) {
+			warning_critical = TRUE;
+		} else if (percentage <= BATTERY_VERY_LOW_PERCENTAGE) {
+			warning_very_low = TRUE;
+		} else if (percentage <= BATTERY_LOW_PERCENTAGE) {
+			warning_low = TRUE;
+		}
+	} else {
+		if (remaining_time <= BATTERY_CRITICAL_REMAINING_TIME) {
+			warning_critical = TRUE;
+		} else if (remaining_time <= BATTERY_VERY_LOW_REMAINING_TIME) {
+			warning_very_low = TRUE;
+		} else if (remaining_time <= BATTERY_LOW_REMAINING_TIME) {
+			warning_low = TRUE;
+		}
+	}
 
-	/* less than critical, do action */
-	if (percentage < critical_threshold) {
-		g_debug ("battery is below critical limit!");
-		manager_policy_do (manager, GPM_PREF_BATTERY_CRITICAL);
-
-		return;
+	/* we only do the notifications once, and we don't want lots of
+	 * notifications on startup if battery is very low */
+	if (manager->priv->done_warning_critical) {
+		warning_critical = FALSE;
+		warning_very_low = FALSE;
+		warning_low = FALSE;
+	}
+	if (manager->priv->done_warning_very_low) {
+		warning_very_low = FALSE;
+		warning_low = FALSE;
+	}
+	if (manager->priv->done_warning_low) {
+		warning_low = FALSE;
 	}
 
 	/* critical warning */
-	if (percentage == critical_threshold) {
-		g_debug ("battery is critical limit!");
-		remaining = get_timestring_from_minutes (minutes);
-
+	if (warning_critical) {
+		manager->priv->done_warning_critical = TRUE;
+		remaining = gpm_get_timestring (remaining_time);
 		message = g_strdup_printf (_("You have approximately <b>%s</b> "
 					     "of remaining battery life (%d%%). "
 					     "Plug in your AC Adapter to avoid losing data."),
 					   remaining, percentage);
-
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      5000,
 				      _("Battery Critically Low"),
@@ -613,31 +686,46 @@ maybe_notify_battery_power_changed (GpmManager         *manager,
 				      message);
 		g_free (message);
 		g_free (remaining);
-
-		return;
+		goto done;
 	}
 
-	/* low warning */
-	if (percentage < low_threshold) {
-		g_debug ("battery is low!");
-		remaining = get_timestring_from_minutes (minutes);
-		g_assert (remaining);
-
-		message = g_strdup_printf (_("You have approximately <b>%s</b> of remaining battery life (%d%%). "
+	if (warning_very_low) {
+		manager->priv->done_warning_very_low = TRUE;
+		remaining = gpm_get_timestring (remaining_time);
+		message = g_strdup_printf (_("You have approximately <b>%s</b> "
+					     "of remaining battery life (%d%%). "
 					     "Plug in your AC Adapter to avoid losing data."),
 					   remaining, percentage);
+		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+				      5000,
+				      _("Battery Very Low"),
+				      NULL,
+				      message);
+		g_free (message);
+		g_free (remaining);
+		goto done;
+	}
 
+	if (warning_low) {
+		manager->priv->done_warning_low = TRUE;
+		remaining = gpm_get_timestring (remaining_time);
+		message = g_strdup_printf (_("You have approximately <b>%s</b> "
+					     "of remaining battery life (%d%%). "
+					     "Plug in your AC Adapter to avoid losing data."),
+					   remaining, percentage);
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      5000,
 				      _("Battery Low"),
 				      NULL,
 				      message);
-
 		g_free (message);
 		g_free (remaining);
-
-		return;
+		goto done;
 	}
+
+done:
+	/* update icon */
+	tray_icon_update (manager);
 }
 
 gboolean
@@ -697,7 +785,10 @@ gpm_manager_get_dpms_mode (GpmManager  *manager,
 void
 gpm_manager_shutdown (GpmManager *manager)
 {
-	if (! _gpm_manager_can_shutdown (manager)) {
+	gboolean allowed;
+
+	gpm_manager_can_shutdown (manager, &allowed, NULL);
+	if (! allowed) {
 		g_warning ("Cannot shutdown");
 		return;
 	}
@@ -722,22 +813,27 @@ void
 gpm_manager_hibernate (GpmManager *manager)
 {
 	gboolean should_lock = gpm_screensaver_lock_enabled ();
+	gboolean allowed;
 
-	if (! _gpm_manager_can_hibernate (manager)) {
+	gpm_manager_can_hibernate (manager, &allowed, NULL);
+
+	if (! allowed) {
 		g_warning ("Cannot hibernate");
 		return;
 	}
 
-	if (should_lock)
+	if (should_lock) {
 		gpm_screensaver_lock ();
+	}
 
 	gpm_networkmanager_sleep ();
 	gpm_hal_hibernate ();
 	gpm_networkmanager_wake ();
 
 	/* Poke GNOME ScreenSaver so the dialogue is displayed */
-	if (should_lock)
+	if (should_lock) {
 		gpm_screensaver_poke ();
+	}
 }
 
 /** Do a suspend with all the associated callbacks and methods.
@@ -753,22 +849,27 @@ void
 gpm_manager_suspend (GpmManager *manager)
 {
 	gboolean should_lock = gpm_screensaver_lock_enabled ();
+	gboolean allowed;
 
-	if (! _gpm_manager_can_suspend (manager)) {
+	gpm_manager_can_suspend (manager, &allowed, NULL);
+
+	if (! allowed) {
 		g_warning ("Cannot suspend");
 		return;
 	}
 
-	if (should_lock)
+	if (should_lock) {
 		gpm_screensaver_lock ();
+	}
 
 	gpm_networkmanager_sleep ();
 	gpm_hal_suspend (0);
 	gpm_networkmanager_wake ();
 
 	/* Poke GNOME ScreenSaver so the dialogue is displayed */
-	if (should_lock)
+	if (should_lock) {
 		gpm_screensaver_poke ();
+	}
 }
 
 /** Callback for the idle function.
@@ -922,9 +1023,9 @@ power_button_pressed_cb (GpmPower   *power,
 		}
 
 		if (strcmp (details, "BrightnessUp") == 0) {
-			gpm_hal_set_brightness_up ();
+			gpm_brightness_level_up (manager->priv->brightness);
 		} else if (strcmp (details, "BrightnessDown") == 0) {
-			gpm_hal_set_brightness_down ();
+			gpm_brightness_level_down (manager->priv->brightness);
 		} else if (strcmp (details, "Suspend") == 0) {
 			gpm_manager_suspend (manager);
 		} else if (strcmp (details, "Hibernate") == 0) {
@@ -935,6 +1036,15 @@ power_button_pressed_cb (GpmPower   *power,
 	}
 }
 
+static void
+invalidate_notification_warnings (GpmManager *manager)
+{
+	g_debug ("invalidate_notification_warnings: Done");
+	manager->priv->done_warning_critical = FALSE;
+	manager->priv->done_warning_very_low = FALSE;
+	manager->priv->done_warning_low = FALSE;
+}
+
 static void
 power_on_ac_changed_cb (GpmPower   *power,
 			gboolean    on_ac,
@@ -945,6 +1055,9 @@ power_on_ac_changed_cb (GpmPower   *power,
 	maybe_notify_on_ac_changed (manager, on_ac);
 	change_power_policy (manager, on_ac);
 
+	/* invalidate last warnings */
+	invalidate_notification_warnings (manager);
+
 	g_signal_emit (manager, signals [ON_AC_CHANGED], 0, on_ac);
 }
 
@@ -952,19 +1065,37 @@ static void
 power_battery_power_changed_cb (GpmPower           *power,
 				const char         *kind,
 				int	            percentage,
-				glong	            minutes,
+				int	            remaining_time,
 				gboolean            discharging,
 				gboolean            charging,
 				gboolean            percentagechanged,
 				GpmManager         *manager)
 {
+	gboolean primary;
+
 	maybe_notify_battery_power_changed (manager,
 					    kind,
 					    percentage,
-					    minutes,
+					    remaining_time,
 					    discharging,
 					    charging,
 					    percentagechanged);
+
+	primary = (strcmp (kind, "primary") == 0);
+
+	/* less than critical, do action */
+	/* FIXME, we should probably give the user a chance to cancel */
+	if (discharging && primary && (percentage < BATTERY_CRITICAL_PERCENTAGE)) {
+		g_warning ("Battery is below critical limit!");
+		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+				      5000,
+				      _("Critical action"),
+				      NULL,
+				      _("The battery is below the critical level and "
+					"this computer is about to shutdown."));
+		g_usleep (1000 * 1000 * 5);
+		manager_policy_do (manager, GPM_PREF_BATTERY_CRITICAL);
+	}
 }
 
 static void
@@ -1044,6 +1175,7 @@ callback_gconf_key_changed (GConfClient *client,
 			    gpointer	 user_data)
 {
 	gint	    value = 0;
+	gint	    brightness;
 	GpmManager *manager = GPM_MANAGER (user_data);
 	gboolean    on_ac;
 
@@ -1085,6 +1217,20 @@ callback_gconf_key_changed (GConfClient *client,
 
 		sync_dpms_policy (manager);
 
+	} else if (strcmp (entry->key, GPM_PREF_AC_BRIGHTNESS) == 0) {
+
+		if (on_ac) {
+			brightness = gconf_client_get_int (client, GPM_PREF_AC_BRIGHTNESS, NULL);
+			gpm_brightness_level_set (manager->priv->brightness, brightness);
+		}
+
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_BRIGHTNESS) == 0) {
+
+		if (! on_ac) {
+			brightness = gconf_client_get_int (client, GPM_PREF_BATTERY_BRIGHTNESS, NULL);
+			gpm_brightness_level_set (manager->priv->brightness, brightness);
+		}
+
 	}
 }
 
@@ -1121,10 +1267,10 @@ gpm_manager_setup_tray_icon (GpmManager *manager,
 	g_debug ("creating new tray icon");
 	manager->priv->tray_icon = gpm_tray_icon_new ();
 
-	enabled = _gpm_manager_can_suspend (manager);
+	gpm_manager_can_suspend (manager, &enabled, NULL);
 	gpm_tray_icon_enable_suspend (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
 
-	enabled = _gpm_manager_can_hibernate (manager);
+	gpm_manager_can_hibernate (manager, &enabled, NULL);
 	gpm_tray_icon_enable_hibernate (GPM_TRAY_ICON (manager->priv->tray_icon), enabled);
 
 	g_signal_connect_object (G_OBJECT (manager->priv->tray_icon),
@@ -1154,6 +1300,7 @@ gpm_manager_setup_tray_icon (GpmManager *manager,
 static void
 gpm_manager_init (GpmManager *manager)
 {
+	gboolean on_ac;
 	manager->priv = GPM_MANAGER_GET_PRIVATE (manager);
 
 	manager->priv->gconf_client = gconf_client_get_default ();
@@ -1178,16 +1325,28 @@ gpm_manager_init (GpmManager *manager)
 				 NULL,
 				 NULL);
 
+	manager->priv->brightness = gpm_brightness_new ();
+
 	manager->priv->idle = gpm_idle_new ();
 	g_signal_connect (manager->priv->idle, "changed",
 			  G_CALLBACK (idle_changed_cb), manager);
 
 	manager->priv->dpms = gpm_dpms_new ();
+
+	/* coldplug so we are in the correct state at startup */
 	sync_dpms_policy (manager);
+	tray_icon_update (manager);
+	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
+	change_power_policy (manager, on_ac);
+
 	g_signal_connect (manager->priv->dpms, "mode-changed",
 			  G_CALLBACK (dpms_mode_changed_cb), manager);
 
-	tray_icon_update (manager);
+	invalidate_notification_warnings (manager);
+
+	/* We can change this easily if	this doesn't work in real-world
+	 * conditions, or perhaps make this a gconf configurable. */
+	manager->priv->use_time_to_notify = TRUE;
 }
 
 static void
@@ -1218,6 +1377,10 @@ gpm_manager_finalize (GObject *object)
 		g_object_unref (manager->priv->power);
 	}
 
+	if (manager->priv->brightness != NULL) {
+		g_object_unref (manager->priv->brightness);
+	}
+
 	if (manager->priv->tray_icon != NULL) {
 		gtk_widget_destroy (GTK_WIDGET (manager->priv->tray_icon));
 	}
