@@ -1,3 +1,4 @@
+#include "gpm-marshal.h"
 
 #include	<glib-object.h>
 
@@ -47,7 +48,7 @@
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* NONE:INT,BOXED (src/gpm-marshal.list:1) */
+/* NONE:INT,BOXED (gpm-marshal.list:1) */
 void
 gpm_marshal_VOID__INT_BOXED (GClosure     *closure,
                              GValue       *return_value,
@@ -84,7 +85,7 @@ gpm_marshal_VOID__INT_BOXED (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,STRING (src/gpm-marshal.list:2) */
+/* NONE:STRING,STRING (gpm-marshal.list:2) */
 void
 gpm_marshal_VOID__STRING_STRING (GClosure     *closure,
                                  GValue       *return_value,
@@ -121,7 +122,7 @@ gpm_marshal_VOID__STRING_STRING (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING,STRING,BOOLEAN (src/gpm-marshal.list:3) */
+/* NONE:STRING,STRING,BOOLEAN (gpm-marshal.list:3) */
 void
 gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
                                          GValue       *return_value,
@@ -160,9 +161,9 @@ gpm_marshal_VOID__STRING_STRING_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* NONE:STRING (src/gpm-marshal.list:4) */
+/* NONE:STRING (gpm-marshal.list:4) */
 
-/* NONE:INT,LONG,BOOLEAN,BOOLEAN (src/gpm-marshal.list:5) */
+/* NONE:INT,LONG,BOOLEAN,BOOLEAN (gpm-marshal.list:5) */
 void
 gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN (GClosure     *closure,
                                             GValue       *return_value,
@@ -203,7 +204,7 @@ gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* NONE:INT,LONG,BOOLEAN,BOOLEAN,BOOLEAN,BOOLEAN (src/gpm-marshal.list:6) */
+/* NONE:INT,LONG,BOOLEAN,BOOLEAN,BOOLEAN,BOOLEAN (gpm-marshal.list:6) */
 void
 gpm_marshal_VOID__INT_LONG_BOOLEAN_BOOLEAN_BOOLEAN_BOOLEAN (GClosure     *closure,
                                                             GValue       *return_value,
