@@ -69,6 +69,8 @@ struct GpmTrayIconPrivate
 	gboolean        can_suspend;
 	gboolean        can_hibernate;
 
+	gboolean        embedded;
+
 #if (LIBNOTIFY_VERSION_MINOR == 2)
 	NotifyHandle   *notify;
 #elif (LIBNOTIFY_VERSION_MINOR >= 3)
@@ -174,11 +176,20 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 			
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-	g_return_if_fail (stock_id != NULL);
 
-	gtk_image_set_from_stock (GTK_IMAGE (icon->priv->image),
-				  stock_id,
-				  GTK_ICON_SIZE_LARGE_TOOLBAR);
+	if (stock_id) {
+		gtk_image_set_from_stock (GTK_IMAGE (icon->priv->image),
+					  stock_id,
+					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+	} else {
+		/* FIXME: gtk_image_clear requires gtk 2.8, so until we
+		 * depend on more then 2.6 (required for FC4) we have to
+		 * comment it out
+		gtk_image_clear (GTK_IMAGE (icon->priv->image));*/
+		if (GTK_WIDGET_VISIBLE (icon->priv->image)) {
+			gtk_widget_queue_resize (GTK_WIDGET (icon->priv->image));
+		}
+	}
 }
 
 static void
@@ -226,6 +237,7 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 	const char *authors[] = {
 		"Richard Hughes <richard@hughsie.com>",
 		"William Jon McCann <mccann@jhu.edu>",
+		"Jaap A. Haitsma <jaap@haitsma.org>",
 		NULL};
 	const char *documenters[] = {
 		"Richard Hughes <richard@hughsie.com>",
@@ -318,23 +330,15 @@ gpm_tray_icon_button_press_cb (GtkWidget      *widget,
 {
 	GtkWidget *popup;
 
-	if (event->type == GDK_2BUTTON_PRESS) {
-		gpm_tray_icon_show_preferences_cb (NULL, icon);
-
-		return TRUE;
-	} else if (event->button == 3) {
-		popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
-						   "/GpmTrayPopup");
-		gtk_menu_set_screen (GTK_MENU (popup),
-				     gtk_widget_get_screen (GTK_WIDGET (icon)));
-		gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
-				tray_popup_position_menu, widget,
-				2,
-				gtk_get_current_event_time ());
-		return TRUE;
-	}
-
-	return FALSE;
+	popup = gtk_ui_manager_get_widget (GTK_UI_MANAGER (icon->priv->ui_manager),
+					   "/GpmTrayPopup");
+	gtk_menu_set_screen (GTK_MENU (popup),
+			     gtk_widget_get_screen (GTK_WIDGET (icon)));
+	gtk_menu_popup (GTK_MENU (popup), NULL, NULL,
+			tray_popup_position_menu, widget,
+			2,
+			gtk_get_current_event_time ());
+	return TRUE;
 }
 
 static void
@@ -433,10 +437,24 @@ gpm_tray_icon_constructor (GType                  type,
 	return G_OBJECT (tray);
 }
 
+static void
+gpm_tray_icon_embedded (GtkPlug *plug)
+{
+	GpmTrayIcon *tray;
+
+	tray = GPM_TRAY_ICON (plug);
+
+	tray->priv->embedded = TRUE;
+
+	if (GTK_PLUG_CLASS (parent_class)->embedded)
+		GTK_PLUG_CLASS (parent_class)->embedded (plug);
+}
+
 static void
 gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
+	GtkPlugClass   *plug_class = GTK_PLUG_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -445,6 +463,8 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 	object_class->set_property = gpm_tray_icon_set_property;
         object_class->constructor  = gpm_tray_icon_constructor;
 
+	plug_class->embedded       = gpm_tray_icon_embedded;
+
 	g_type_class_add_private (klass, sizeof (GpmTrayIconPrivate));
 
         signals [SUSPEND] =
@@ -489,9 +509,8 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
                                  G_CALLBACK (gpm_tray_icon_button_press_cb),
                                  icon, 0);
 
+	icon->priv->image = gtk_image_new ();
 
-	icon->priv->image = gtk_image_new_from_stock (GPM_STOCK_AC_ADAPTER,
-						      GTK_ICON_SIZE_LARGE_TOOLBAR);
         gtk_container_add (GTK_CONTAINER (icon->priv->ebox), icon->priv->image);
 
         gtk_container_add (GTK_CONTAINER (icon), icon->priv->ebox);
@@ -715,7 +734,12 @@ gpm_tray_icon_notify (GpmTrayIcon *icon,
 		g_debug ("ignoring notification: %s", primary);
 		return;
 	}
-	
+
+	if (! icon->priv->embedded) {
+		g_debug ("Not embedded in a tray - ignoring notification");
+		return;
+	}
+
 	g_debug ("doing notify: %s", primary);
 
 	libnotify_event (icon,
